from bobbles import *

import random
import operator


from quadtree import *


uniformCol = (60.0/255.0,50.0/255.0,255.0/255.0)
selectedCol = (180.0/255.0,150.0/255.0,200.0/255.0)


class PoliceAI(BobbleAI):
  def __init__(self,pos):
    BobbleAI.__init__(self,pos)

    # Make 'em a little bit variables, and significantly stronger than protestors...
    self.setStrength(random.uniform(2.0,3.0),random.uniform(2.0,3.0))
    self.setSpeed(random.uniform(1.1,1.3),random.uniform(1.1,1.3))

  def step(self,body):
    # Update its position in the database, so selection can work...
    newPos = body.getPosition()
    self.manager.qt.update(self,newPos[0],newPos[1])
    
    # Safety check - if its fallen out of the level destroy...
    if newPos[2]<-100.0:
      self.die()
      
    return BobbleAI.step(self,body)

  def die(self):
    self.manager.qt.remove(self)
    BobbleAI.die(self)



class PoliceMan:
  def __init__(self,bobbles,stats):
    self.bobbles = bobbles
    self.stats = stats

    # Create the quad tree that contains policemen - needed so we can select them quickly...
    self.qt = QuadTree(-200.0,200.0,-200.0,200.0)

    # List of selected policemen...
    self.selected = []

    # Load the police deployment info, deploy them at the various locations required...
    locs = loader.loadModel('data/locations')
    nodes = locs.findAllMatches('**/=IsA=Deployment')
    for i in xrange(nodes.size()):
      node = nodes[i]
      width = int(node.getTag('Width'))
      height = int(node.getTag('Height'))
      for yo in xrange(height):
        for xo in xrange(width):
          self.addPolice(render.getRelativePoint(node,Vec3(xo - 0.5*width,yo - 0.5*height,0.0)))

  def addPolice(self,pos):
    # Create the bobble, store in db...
    np = self.bobbles.addBobble(pos,0.0,uniformCol,PoliceAI)
    self.qt.update(np,pos[0],pos[1])

    # Give it a pointer to this...
    np.manager = self
    self.stats.recordPoliceman()


  def selectPoint(self,point):
    """ Given a (x,y) in the ground plane this selects the relevant policemen."""
    self.selected = filter(lambda x: not x.mesh.isEmpty(),self.selected) # Safety - some could of been deleted.
    
    for pm in self.selected:
      pm.setCol(uniformCol)

    self.selected = []
    
    for pm in self.qt.inCircle(point[0],point[1],0.5):
      if not pm.mesh.isEmpty(): # No idea why this is needed.
        pm.setCol(selectedCol)
        self.selected.append(pm)
  
  def selectRegion(self,poly):
    """ Given a list of (x,y) tuples defining a polygon in the ground plane this selects the relevant policemen."""
    self.selected = filter(lambda x: not x.mesh.isEmpty(),self.selected) # Safety - some could of been deleted.
    
    for pm in self.selected:
      pm.setCol(uniformCol)

    self.selected = []

    for pm in self.qt.inPolygon(poly,0.5):
      if not pm.mesh.isEmpty(): # No idea why this is needed.
        pm.setCol(selectedCol)
        self.selected.append(pm)


  def move(self,a,b):
    """Given a pair of (x,y) tuples moves the policemen to this location - has them form a line, many deep if need be. If a depth limit is passed it treats it as a point and just makes them congregate."""
    self.selected = filter(lambda x: not x.mesh.isEmpty(),self.selected) # Safety - some could of been deleted.
    
    n = (b[0]-a[0],b[1]-a[1])
    if n[0]**2.0 + n[1]**2.0 > 1.0:
      # Reorder selected by projecting to line relative positions...
      self.sortSel = map(lambda pm: (pm,pm.mesh.getX()*n[0]+pm.mesh.getY()*n[1]),self.selected)
      self.sortSel.sort(key=operator.itemgetter(1))
      self.sortSel = map(operator.itemgetter(0),self.sortSel)
      
      # Line has been created - form up...
      for i,pm in enumerate(self.sortSel):
        t = float(i)/float(len(self.selected)-1)
        tx = (1.0-t)*a[0] + t*b[0]
        ty = (1.0-t)*a[1] + t*b[1]
        pm.setDest(tx,ty)
    else:
      tx = 0.5*(a[0]+b[0])
      ty = 0.5*(a[1]+b[1])
      
      for pm in self.selected:
        pm.setDest(tx,ty)