from direct.showbase import DirectObject

from protestor import *

class Stats(DirectObject.DirectObject):
  """Stores how many of each event has happened; also handles injury and victory conditions."""
  def __init__(self):
    # Make all the stat stores...
    self.toiletStreet = 0
    self.toiletUnderwear = 0
    self.gotHungry = 0
    self.starved = 0
    self.dehydrated = 0
    self.escaped = 0
    self.unexplained = 0

    self.totalProtestors = 0
    self.totalPolice = 0

    self.collisions = 0 # Between a policeman and a protestor.
    self.injuries = 0 # Number of collisions that don't kill.
    self.closedDeaths = 0 # Deaths where the policeman was suspended.
    self.openDeaths = 0 # Deaths where the policeman got away with it.

    self.money = 200

    # Status of the law...
    self.waterban = False # Reudces the percentage of protestors with water bottles.
    self.photoBan = False # Reduces the chances of a policeman being caught for killing a protestor.

    # Arrange for the handling of collisions...
    self.accept('collision',self.onCollision)

  def onCollision(self,entry):
    if not entry.getGeom1().hasBody():
      return
    if not entry.getGeom2().hasBody():
      return

    b1 = entry.getBody1()
    b2 = entry.getBody2()
    try:
      ai1 = self.bodyToAI[str(b1)]
      ai2 = self.bodyToAI[str(b2)]
    except:
      return # One has probably already died.
    if ai1.__class__==ai2.__class__: return

    self.collisions += 1

    strength = (b1.getLinearVel() + b2.getLinearVel()).length()
    if strength > 3.0:
      if ai1.__class__==ProtestorAI:
        pr = ai1
        pm = ai2
      else:
        pr = ai2
        pm = ai1

      if pr.injured:
        if random.random()<0.05:
          # Protestor is already injured - second hit means death and the policeman being suspended, unless photography has been banned...
          pr.die()
          if (self.photoBan and random.random()<0.01) or (not self.photoBan and random.random()<0.1):
            # Policeman is seen - they are suspended...
            self.closedDeaths += 1
            pm.die()
          else:
            # Insufficient evidence is avaliable to prosecute...
            self.openDeaths += 1
            self.onDeath()
      else:
        if random.random()<0.25:
          self.injuries += 1
          pr.injured = True
          c = pr.mesh.getColor()
          c[0] = 1.0
          pr.mesh.setColor(c)



  def onDeath(self):
    """Updates the money avaliable after a death."""
    self.money += self.totalPolice - self.closedDeaths

  def recordEscape(self):
    self.escaped += 1

  def recordDehydration(self):
    self.dehydrated += 1
    self.onDeath()

  def recordStarvation(self):
    self.starved += 1
    self.onDeath()

  def recordHunger(self):
    self.gotHungry += 1

  def recordDampUnderwear(self):
    self.toiletUnderwear += 1

  def recordDampStreet(self):
    self.toiletStreet += 1

  def recordUnexplained(self):
    self.unexplained += 1

  def recordArrival(self):
    self.totalProtestors += 1

  def recordPoliceman(self):
    self.totalPolice += 1